/**
 * lazy-hydration v0.2.2
 * (c) 2019 Rahul Kadyan <hi@znck.me> (https://znck.me)
 * @license MIT
 * @description Built with rollup-plugin-vue v4.6.1 and vue-template-compiler v2.5.22
 */
'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var PropTypes = _interopDefault(require('@znck/prop-types'));

var isBrowser = typeof window !== "undefined";
var io = typeof IntersectionObserver !== "undefined" ? new IntersectionObserver(function (entries) {
  entries.forEach(function (entry) {
    if (entry.isIntersecting || entry.intersectionRatio > 0) {
      entry.target.parentElement.hydrate && entry.target.parentElement.hydrate();
    }
  });
}) : null;
var script = {
  props: process.env.NODE_ENV !== 'production' ? {
    on: PropTypes.oneOfType(String, PropTypes.arrayOf(String)),
    onClick: PropTypes.bool,
    onHover: PropTypes.bool,
    onInteraction: PropTypes.bool,
    whenVisible: PropTypes.bool,
    whenIdle: PropTypes.bool,
    withDelay: PropTypes.number,
    ssrOnly: PropTypes.bool,
    force: PropTypes.bool
  } : ["on", "on", "onClick", "onHover", "onInteraction", "whenVisible", "whenIdle", "withDelay", "ssrOnly"],
  data: function data() {
    return {
      hydrated: !isBrowser
    };
  },
  created: function created() {
    var _this = this;

    if (process.env.NODE_ENV !== 'production') {
      PropTypes.validate(function () {
        if (!_this.on && !_this.onClick && !_this.onHover && !_this.onInteraction && !_this.whenVisible && !_this.whenIdle && !_this.ssrOnly && _this.force === undefined) {
          console.error("Select at least one trigger to enable hydration. If you don't want to hydrate at all use 'ssr-only'.");
        }

        if (_this.withDelay && _this.withDelay < 800) {
          console.warn("Delay duration ".concat(_this.withDelay, "ms is too low. A good choice would be around 2000ms. See https://github.com/znck/lazy-hydration."));
        }
      });
    }
  },
  mounted: function mounted() {
    var _this2 = this;

    if (this.$el.childElementCount === 0) {
      // No SSR rendered content. Render now.
      this.hydrate();
      return;
    }

    if (this.ssrOnly) return;
    var withDelay;
    var on = this.on ? [this.on].flat() : [];

    if (this.onClick) {
      on.push("click");
    }

    if (this.onHover || this.onInteraction) {
      on.push("mouseenter");

      if (this.onInteraction) {
        on.push("focus");
      }
    }

    if (this.whenIdle) {
      if (typeof requestIdleCallback !== "undefined") {
        var id = requestIdleCallback(function () {
          requestAnimationFrame(function () {
            _this2.hydrate();
          });
        }, {
          timeout: 500
        });

        this.idle = function () {
          return cancelIdleCallback(id);
        };
      } else withDelay = 2000;
    }

    if (this.whenVisible) {
      // As root node does not have any box model, it cannot intersect.
      var el = this.$el.children[0];
      el.hydrate = this.hydrate;
      if (io) io.observe(el);else {
        withDelay = 2000;

        if (process.env.NODE_ENV !== 'production') {
          PropTypes.validate(function () {
            return console.warn("IntersectionObserver polyfill is required.");
          });
        }
      }

      this.visible = function () {
        io && io.unobserve(el);
        delete el.hydrate;
      };
    }

    if (on.length) {
      on.forEach(function (event) {
        return _this2.$el.addEventListener(event, _this2.hydrate, {
          capture: true,
          once: true
        });
      });

      this.off = function () {
        return on.forEach(function (event) {
          return _this2.$el.removeEventListener(event, _this2.hydrate);
        });
      };
    }

    if (this.withDelay || withDelay) {
      var _id = setTimeout(this.hydrate, this.withDelay || withDelay);

      this.delay = function () {
        return clearTimeout(_id);
      };
    }
  },
  beforeDestroy: function beforeDestroy() {
    this.cleanup();
  },
  methods: {
    cleanup: function cleanup() {
      var handlers = ["visible", "idle", "delay", "off"];

      for (var _i = 0; _i < handlers.length; _i++) {
        var handler = handlers[_i];

        if (handler in this) {
          this[handler]();
          delete this[handler];
        }
      }
    },
    hydrate: function hydrate() {
      this.hydrated = true;
      this.cleanup();
    }
  },
  render: function render(h) {
    var vnode = this.hydrated ? h("div", {
      staticStyle: "display: contents"
    }, this.$slots.default) : h("div");

    if (isBrowser) {
      vnode.asyncFactory = this.hydrated ? {
        resolved: true
      } : {};
      vnode.isAsyncPlaceholder = !this.hydrated;
    }

    return vnode;
  },
  watch: {
    force: {
      handler: function handler(value) {
        if (value) this.hydrate();
      },
      immediate: true
    }
  }
};

function normalizeComponent(compiledTemplate, injectStyle, defaultExport, scopeId, isFunctionalTemplate, moduleIdentifier
/* server only */
, isShadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
  if (typeof isShadowMode === 'function') {
    createInjectorSSR = createInjector;
    createInjector = isShadowMode;
    isShadowMode = false;
  } // Vue.extend constructor export interop


  const options = typeof defaultExport === 'function' ? defaultExport.options : defaultExport; // render functions

  if (compiledTemplate && compiledTemplate.render) {
    options.render = compiledTemplate.render;
    options.staticRenderFns = compiledTemplate.staticRenderFns;
    options._compiled = true; // functional template

    if (isFunctionalTemplate) {
      options.functional = true;
    }
  } // scopedId


  if (scopeId) {
    options._scopeId = scopeId;
  }

  let hook;

  if (moduleIdentifier) {
    // server build
    hook = function (context) {
      // 2.3 injection
      context = context || // cached call
      this.$vnode && this.$vnode.ssrContext || // stateful
      this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext; // functional
      // 2.2 with runInNewContext: true

      if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
        context = __VUE_SSR_CONTEXT__;
      } // inject component styles


      if (injectStyle) {
        injectStyle.call(this, createInjectorSSR(context));
      } // register component module identifier for async chunk inference


      if (context && context._registeredComponents) {
        context._registeredComponents.add(moduleIdentifier);
      }
    }; // used by ssr in case component is cached and beforeCreate
    // never gets called


    options._ssrRegister = hook;
  } else if (injectStyle) {
    hook = isShadowMode ? function () {
      injectStyle.call(this, createInjectorShadow(this.$root.$options.shadowRoot));
    } : function (context) {
      injectStyle.call(this, createInjector(context));
    };
  }

  if (hook) {
    if (options.functional) {
      // register for functional component in vue file
      const originalRender = options.render;

      options.render = function renderWithStyleInjection(h, context) {
        hook.call(context);
        return originalRender(h, context);
      };
    } else {
      // inject component registration as beforeCreate hook
      const existing = options.beforeCreate;
      options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
    }
  }

  return defaultExport;
}

/* script */
const __vue_script__ = script;
// For security concerns, we use only base name in production mode. See https://github.com/vuejs/rollup-plugin-vue/issues/258
script.__file = "Hydrate.vue";

/* template */

  /* style */
  const __vue_inject_styles__ = undefined;
  /* scoped */
  const __vue_scope_id__ = undefined;
  /* module identifier */
  const __vue_module_identifier__ = "data-v-02894aca";
  /* functional template */
  const __vue_is_functional_template__ = undefined;
  /* style inject */
  
  /* style inject SSR */
  

  
  var Hydrate = normalizeComponent(
    {},
    __vue_inject_styles__,
    __vue_script__,
    __vue_scope_id__,
    __vue_is_functional_template__,
    __vue_module_identifier__,
    undefined,
    undefined
  );

module.exports = Hydrate;
